#!/usr/bin/env python
#----------------------------------------------------------------------------
# Name:         PyTAS.py
# Purpose:      A Python-based Tile Assembly Simulator
#
# Author:       Matthew Patitz
#
# Created:      
# RCS-ID:       
# Copyright:    
# License:      
#----------------------------------------------------------------------------

"""
This simulator is designed to simulate abstract Tile Assembly Model (aTAM) systems
based on the ISU TAS format.

For a list of requirements and dependencies, as well as instructions for installing
and running PyTAS, please see the README.txt document.

pip install PyPubSub

"""

#
#Previous dependencies not currently used:
#    pyPdf2      (install with 'pip install pypdf2')
#    reportlab   (install with 'pip install reportlab')


#import wxversion
#wxversion.select('3.0-msw', optionsRequired=True)
#wxversion.select('3.0-Phoenix', optionsRequired=True)
import wx
import wx.richtext as rt
from PyTASAssembly import *
from PyTASCanvas import *
from SimulatorFramework.SimApp import *
from SimulatorFramework import SimUtils
from PyTAS3DCanvas import *
from SimulatorFramework.TileTypeEditor import TileTypeEditorFrame
from version import PyTASVersion
from SimulatorFramework.version import SimAppVersion
import wx.lib.agw.cubecolourdialog as CCD
import wx.aui
import wx.lib.intctrl
import wx.lib.masked.numctrl


#===============================================================================
# from reportlab.lib.pagesizes import letter
# from reportlab.lib.units import inch
# from reportlab.pdfgen import canvas
# from reportlab.platypus import SimpleDocTemplate, Image
#===============================================================================

#---------------------------------------------------------------------------

class SetTemperatureDialog(wx.Dialog):
    def __init__(self, parent, ID=-1, size=wx.DefaultSize, pos=wx.DefaultPosition, style=wx.DEFAULT_DIALOG_STYLE):
        self.parent = parent
        title = "Set temperature"
        wx.Dialog.__init__(self, parent, ID, title, pos, size, style)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.AddSpacer(5)
        label = wx.StaticText(self, -1, "Set the temperature parameter for the tile   \nassembly system (as a positive integer)")
        sizer.Add(label, 0, wx.ALIGN_LEFT|wx.ALL, 5)
        sizer.AddSpacer(10)

        self.tempSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.tempSizer.AddSpacer(5)
        lblTemp = wx.StaticText(self, label="Temperature:")
        self.tempSizer.Add(lblTemp, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        self.tempSizer.AddSpacer(5)    
        self.tempCtrl = wx.lib.intctrl.IntCtrl(self, -1, int(self.parent.assembly.temperature), wx.DefaultPosition, wx.Size(50,25))
        self.tempCtrl.SetMin(1)
        self.tempCtrl.SetNoneAllowed(False)
        self.tempSizer.Add(self.tempCtrl)
        
        sizer.Add(self.tempSizer)
        sizer.AddSpacer(10)

        btnsizer = wx.StdDialogButtonSizer()
        
        btn = wx.Button(self, wx.ID_OK)
        btn.SetDefault()
        btnsizer.AddButton(btn)

        btn = wx.Button(self, wx.ID_CANCEL)
        btnsizer.AddButton(btn)
        btnsizer.Realize()

        sizer.Add(btnsizer, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, 5)

        self.SetSizer(sizer)
        sizer.Fit(self)
        
#---------------------------------------------------------------------------

class SetOutlineDialog(wx.Dialog):
    def __init__(self, parentApp):
        wx.Dialog.__init__(self, parentApp.frame, -1, "Set outline details", style=wx.DEFAULT_DIALOG_STYLE)

        self.parentApp = parentApp

        self.outlinePanel = scrolled.ScrolledPanel(self, wx.ID_ANY, pos=wx.DefaultPosition, style = wx.TAB_TRAVERSAL)

        self.mainSizer = wx.BoxSizer(wx.VERTICAL)
        self.mainSizer.AddSpacer(5)

        self.topSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.botSizer = wx.BoxSizer(wx.HORIZONTAL)

        self.olCtrl = wx.CheckBox(self, -1, "Draw outlines", wx.DefaultPosition, wx.DefaultSize)
        self.widthCtrl = wx.lib.masked.numctrl.NumCtrl(self, -1, 1, wx.DefaultPosition, wx.DefaultSize)
        self.distCtrl = wx.lib.intctrl.IntCtrl(self, -1, 1000, wx.DefaultPosition, wx.DefaultSize)
        self.distCtrl.SetNoneAllowed(False)

        self.olCtrl.SetValue(self.parentApp.canvas.graphics.drawOutlines)
        self.widthCtrl.SetValue(int(self.parentApp.canvas.graphics.outlineWidth))
        self.distCtrl.SetValue(int(self.parentApp.canvas.graphics.outlineDistance))

        self.topSizer.Add(self.olCtrl)
        self.topSizer.AddSpacer(5)
        lblW = wx.StaticText(self, label="Outline width:")
        self.topSizer.Add(lblW, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        self.topSizer.Add(self.widthCtrl, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        self.topSizer.AddSpacer(5)
        lblD = wx.StaticText(self, label="Render distance:")
        self.topSizer.Add(lblD, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        self.topSizer.Add(self.distCtrl, 0, wx.ALIGN_CENTER_VERTICAL, 5)

        self.okButton = wx.Button(self, -1, "Ok", size=(60,25))
        self.okButton.Bind(wx.EVT_BUTTON, self.OnOk)
        self.cancelButton = wx.Button(self, -1, "Cancel", size=(60,25))
        self.cancelButton.Bind(wx.EVT_BUTTON, self.OnCancel)
        self.botSizer.Add(self.okButton)
        self.botSizer.Add(self.cancelButton)

        self.mainSizer.Add(self.topSizer)
        self.mainSizer.AddSpacer(15)
        self.mainSizer.Add(self.botSizer)

        self.SetSizer(self.mainSizer)
        self.mainSizer.Fit(self)

    def OnCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

    def OnOk(self, event):
        self.parentApp.canvas.graphics.setOutlineMode(self.olCtrl.GetValue(), float(self.widthCtrl.GetValue()), float(self.distCtrl.GetValue()))
        self.parentApp.canvas.Refresh()
        self.EndModal(wx.ID_OK)
#---------------------------------------------------------------------------

class CameraPositionWin(wx.Dialog):
    
    def __init__(self, parentApp):
        wx.Dialog.__init__(self, parentApp.frame, -1, "Set camera position", size=(280,250), style=wx.DEFAULT_DIALOG_STYLE)
        
        #self.cameraCenter     = np.array([0.0, 0.0, 0.0])
        #self.cameraUp         = np.array([0.0, 1.0, 0.0])
        #self.cameraPosition   = np.array([0.0, 0.0, 8.0])

        self.parentApp = parentApp
        self.cameraPositionPanel = scrolled.ScrolledPanel(self, wx.ID_ANY, pos=wx.DefaultPosition, style = wx.TAB_TRAVERSAL)
        self.cameraPositionPanelSizer = wx.BoxSizer(wx.VERTICAL)
        
        self.mainSizer = wx.BoxSizer(wx.VERTICAL)
        self.mainSizer.AddSpacer(5)

        # Make sizer for centered coordinates  
        self.topSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.coordsSizer = wx.StaticBoxSizer(wx.HORIZONTAL, self.cameraPositionPanel, "Coordinates to be centered")
        self.coordsSizer.AddSpacer(15)

        lblX = wx.StaticText(self, label="X:")
        self.coordsSizer.Add(lblX, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        self.coordsSizer.AddSpacer(5)    
        self.xCtrl = wx.lib.intctrl.IntCtrl(self.cameraPositionPanel, -1, int(self.parentApp.canvas.cameraCenter[0]), wx.DefaultPosition, wx.Size(50,25))
        self.xCtrl.SetToolTip("Current minimum x = " + str(self.parentApp.canvas.assembly.minTiledX) + ", maximum x = " + str(self.parentApp.canvas.assembly.maxTiledX))
        self.xCtrl.SetNoneAllowed(False)
        
        self.coordsSizer.Add(self.xCtrl)
        self.coordsSizer.AddSpacer(8)
        
        lblY = wx.StaticText(self, label="Y:")
        self.coordsSizer.Add(lblY, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        self.coordsSizer.AddSpacer(5)
        self.yCtrl = wx.lib.intctrl.IntCtrl(self.cameraPositionPanel, -1, int(self.parentApp.canvas.cameraCenter[1]), wx.DefaultPosition, wx.Size(50,25))
        self.yCtrl.SetToolTip("Current minimum y = " + str(self.parentApp.canvas.assembly.minTiledY) + ", maximum y = " + str(self.parentApp.canvas.assembly.maxTiledY))
        self.yCtrl.SetNoneAllowed(False)
        self.coordsSizer.Add(self.yCtrl)
        self.coordsSizer.AddSpacer(8)
        
        lblZ = wx.StaticText(self, label="Z:")
        self.coordsSizer.Add(lblZ, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        self.coordsSizer.AddSpacer(5)
        self.zCtrl = wx.lib.intctrl.IntCtrl(self.cameraPositionPanel, -1, int(self.parentApp.canvas.cameraCenter[2]), wx.DefaultPosition, wx.Size(50,25))
        self.zCtrl.SetToolTip("Current minimum z = " + str(self.parentApp.canvas.assembly.minTiledZ) + ", maximum z = " + str(self.parentApp.canvas.assembly.maxTiledZ))
        self.zCtrl.SetNoneAllowed(False)
        self.coordsSizer.Add(self.zCtrl)
        
        self.topSizer.AddSpacer(10)
        self.topSizer.Add(self.coordsSizer, 0, wx.ALIGN_CENTER_HORIZONTAL)
        self.mainSizer.Add(self.topSizer)
        self.mainSizer.AddSpacer(5)
        
        
        # Make sizer for camera coordinates  
        self.topSizer2 = wx.BoxSizer(wx.HORIZONTAL)
        self.camCoordsSizer = wx.StaticBoxSizer(wx.HORIZONTAL, self.cameraPositionPanel, "Coordinates for the camera")
        self.camCoordsSizer.AddSpacer(15)

        lblX = wx.StaticText(self, label="X:")
        self.camCoordsSizer.Add(lblX, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        self.camCoordsSizer.AddSpacer(5)    
        self.xCamCtrl = wx.lib.intctrl.IntCtrl(self.cameraPositionPanel, -1, int(self.parentApp.canvas.cameraPosition[0]), wx.DefaultPosition, wx.Size(50,25))
        self.xCamCtrl.SetNoneAllowed(False)
        self.camCoordsSizer.Add(self.xCamCtrl)
        self.camCoordsSizer.AddSpacer(8)
        
        lblY = wx.StaticText(self, label="Y:")
        self.camCoordsSizer.Add(lblY, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        self.camCoordsSizer.AddSpacer(5)
        self.yCamCtrl = wx.lib.intctrl.IntCtrl(self.cameraPositionPanel, -1, int(self.parentApp.canvas.cameraPosition[1]), wx.DefaultPosition, wx.Size(50,25))
        self.yCamCtrl.SetNoneAllowed(False)
        self.camCoordsSizer.Add(self.yCamCtrl)
        self.camCoordsSizer.AddSpacer(8)
        
        lblZ = wx.StaticText(self, label="Z:")
        self.camCoordsSizer.Add(lblZ, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        self.camCoordsSizer.AddSpacer(5)
        self.zCamCtrl = wx.lib.intctrl.IntCtrl(self.cameraPositionPanel, -1, int(self.parentApp.canvas.cameraPosition[2]), wx.DefaultPosition, wx.Size(50,25))
        self.zCamCtrl.SetNoneAllowed(False)
        self.camCoordsSizer.Add(self.zCamCtrl)
        
        self.topSizer2.AddSpacer(10)
        self.topSizer2.Add(self.camCoordsSizer, 0, wx.ALIGN_CENTER_HORIZONTAL)
        self.mainSizer.Add(self.topSizer2)
        self.mainSizer.AddSpacer(5)

        self.secondSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.secondSizer.AddSpacer(40)
        
        # Set from self.cameraUp
        bXNeg = False
        if self.parentApp.canvas.cameraUp[0] < 0:
            bXNeg = True
        bXPos = False
        if self.parentApp.canvas.cameraUp[0] > 0:
            bXPos = True
        bYNeg = False
        if self.parentApp.canvas.cameraUp[1] < 0:
            bYNeg = True
        bYPos = False
        if self.parentApp.canvas.cameraUp[1] > 0:
            bYPos = True
        bZNeg = False
        if self.parentApp.canvas.cameraUp[2] < 0:
            bZNeg = True
        bZPos = False
        if self.parentApp.canvas.cameraUp[2] > 0:
            bZPos = True

        self.upBox = wx.StaticBoxSizer(wx.HORIZONTAL, self.cameraPositionPanel, "Upward direction for camera")
        self.innerSizer2 = wx.BoxSizer(wx.HORIZONTAL)
        self.innerSizer2.AddSpacer(15)
        self.upSizer = wx.GridSizer(cols=3, hgap=5, vgap=5)
        self.xNeg2 = wx.RadioButton(self.cameraPositionPanel, -1, "X-", name="radioUp", style=wx.RB_GROUP)
        self.xNeg2.SetValue(bXNeg)
        self.xNeg2.Bind(wx.EVT_RADIOBUTTON, self.OnUp)
        self.upSizer.Add(self.xNeg2)
        self.yNeg2 = wx.RadioButton(self.cameraPositionPanel, -1, "Y-", name="radioUp")
        self.yNeg2.SetValue(bYNeg)
        self.yNeg2.Bind(wx.EVT_RADIOBUTTON, self.OnUp)
        self.upSizer.Add(self.yNeg2)
        self.zNeg2 = wx.RadioButton(self.cameraPositionPanel, -1, "Z-", name="radioUp")
        self.zNeg2.SetValue(bZNeg)
        self.zNeg2.Bind(wx.EVT_RADIOBUTTON, self.OnUp)
        self.upSizer.Add(self.zNeg2)
        self.xPos2 = wx.RadioButton(self.cameraPositionPanel, -1, "X+", name="radioUp")
        self.xPos2.SetValue(bXPos)
        self.xPos2.Bind(wx.EVT_RADIOBUTTON, self.OnUp)
        self.upSizer.Add(self.xPos2)
        self.yPos2 = wx.RadioButton(self.cameraPositionPanel, -1, "Y+", name="radioUp")
        self.yPos2.SetValue(bYPos)
        self.yPos2.Bind(wx.EVT_RADIOBUTTON, self.OnUp)
        self.upSizer.Add(self.yPos2)
        self.zPos2 = wx.RadioButton(self.cameraPositionPanel, -1, "Z+", name="radioUp")
        self.zPos2.SetValue(bZPos)
        self.zPos2.Bind(wx.EVT_RADIOBUTTON, self.OnUp)
        self.upSizer.Add(self.zPos2)
        
        self.innerSizer2.Add(self.upSizer)
        self.upBox.Add(self.innerSizer2)
        self.secondSizer.Add(self.upBox)
        
        self.mainSizer.Add(self.secondSizer)
        self.mainSizer.AddSpacer(10)
        
        self.thirdSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.thirdSizer.AddSpacer(70)
        self.defaultButton = wx.Button(self.cameraPositionPanel, -1, "Default", size=(60,25))
        self.defaultButton.Bind(wx.EVT_BUTTON, self.OnDefault)
        self.thirdSizer.Add(self.defaultButton)
        self.thirdSizer.AddSpacer(5)
        self.okButton = wx.Button(self.cameraPositionPanel, -1, "Ok", size=(60,25))
        self.okButton.Bind(wx.EVT_BUTTON, self.OnOk)
        self.thirdSizer.Add(self.okButton)
        self.thirdSizer.AddSpacer(5)
        self.cancelButton = wx.Button(self.cameraPositionPanel, -1, "Cancel", size=(60,25))
        self.cancelButton.Bind(wx.EVT_BUTTON, self.OnCancel)
        self.thirdSizer.Add(self.cancelButton)
        self.mainSizer.Add(self.thirdSizer)

        self.cameraPositionPanelSizer.Add(self.mainSizer)
 
        self.cameraPositionPanelSizer.Layout()
        self.cameraPositionPanel.SetSizerAndFit(self.cameraPositionPanelSizer)
        self.cameraPositionPanel.SetupScrolling()

    def OnDefault(self, event):
        
        self.xCtrl.SetValue(0)
        self.yCtrl.SetValue(0)
        self.zCtrl.SetValue(0)
        
        self.xCamCtrl.SetValue(0)
        self.yCamCtrl.SetValue(0)
        self.zCamCtrl.SetValue(8)
        
        self.yPos2.SetValue(True)

    def OnUp(self, event):
        return

    def OnOk(self, event):
        # Make sure coords are within assembly bounds
        x = self.xCtrl.GetValue()
        if x < self.parentApp.canvas.assembly.minTiledX:
            wx.MessageDialog(self, "The x coordinate provided is less than the minimum x coordinate in the current assembly (which is " + str(self.parentApp.canvas.assembly.minTiledX) + ").", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return False
        if x > self.parentApp.canvas.assembly.maxTiledX:
            wx.MessageDialog(self, "The x coordinate provided is greater than the maximum x coordinate in the current assembly (which is " + str(self.parentApp.canvas.assembly.maxTiledX) + ").", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return False
        y = self.yCtrl.GetValue()
        if y < self.parentApp.canvas.assembly.minTiledY:
            wx.MessageDialog(self, "The y coordinate provided is less than the minimum y coordinate in the current assembly (which is " + str(self.parentApp.canvas.assembly.minTiledY) + ").", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return False
        if y > self.parentApp.canvas.assembly.maxTiledY:
            wx.MessageDialog(self, "The y coordinate provided is greater than the maximum y coordinate in the current assembly (which is " + str(self.parentApp.canvas.assembly.maxTiledY) + ").", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return False
        z = self.zCtrl.GetValue()
        if z < self.parentApp.canvas.assembly.minTiledZ:
            wx.MessageDialog(self, "The z coordinate provided is less than the minimum z coordinate in the current assembly (which is " + str(self.parentApp.canvas.assembly.minTiledZ) + ").", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return False
        if z > self.parentApp.canvas.assembly.maxTiledZ:
            wx.MessageDialog(self, "The z coordinate provided is greater than the maximum z coordinate in the current assembly (which is " + str(self.parentApp.canvas.assembly.maxTiledZ) + ").", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return False
        
        camX = self.xCamCtrl.GetValue()
        camY = self.yCamCtrl.GetValue()
        camZ = self.zCamCtrl.GetValue()
        
        coords = (float(x), float(y), float(z))
        camCoords = (float(camX), float(camY), float(camZ))
        
        # Set this based on the radio button selection
        up = (0.0, 10.0, 0.0)
        if self.xNeg2.GetValue() == True:
            up = (-10.0, 0.0, 0.0)
        elif self.yNeg2.GetValue() == True:
            up = (0.0, -10.0, 0.0)
        elif self.zNeg2.GetValue() == True:
            up = (0.0, 0.0, -10.0)
        elif self.xPos2.GetValue() == True:
            up = (10.0, 0.0, 0.0)
        elif self.yPos2.GetValue() == True:
            up = (0.0, 10.0, 0.0)
        elif self.zPos2.GetValue() == True:
            up = (0.0, 0.0, 10.0)

        self.parentApp.canvas.CameraLookAt(eye = camCoords, center = coords, up = up)
        self.parentApp.canvas.Refresh()
        self.EndModal(wx.ID_OK)

    def OnCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

#---------------------------------------------------------------------------

class PyTASApp(SimApp):
    def __init__(self):
        self.dimMode = SIMU_2D_MODE
        self.tilesPerStepPanel = None
        SimApp.__init__(self)
        self.frame.Bind(wx.EVT_CHAR, self.OnChar)

    def OnInit(self):
        SimUtils.DocumentationFileList = ["PyTAS Overview", "Getting Started", "TAM", "Load and Save", "Simulator Control", "TAMScript"]

        retVal = SimApp.OnInit(self, title="PyTAS aTAM Simulator")
        self.tileTypeClassName = PyTASTileType
        return retVal

    def CreateCanvas(self):
        if self.dimMode == SIMU_2D_MODE:
            self.canvas = PyTASCanvas(self.frame, wx.ID_ANY, self.frame, wx.DefaultSize, self)
        elif self.dimMode == SIMU_3D_MODE:
            self.canvas = PyTAS3DCanvas(self.frame, wx.ID_ANY, self.frame, wx.DefaultSize, self)
        
    def CreateMenu(self):
        menuBar = wx.MenuBar()
        menu = wx.Menu()

        itemNewSystem = menu.Append(-1, "&New system (seed and tile set)\tCtrl-N", "New tile assembly system (seed and tile set)")
        self.Bind(wx.EVT_MENU, self.OnNewSystem, itemNewSystem)
        item1 = menu.Append(-1, "&Load system  (seed and tile set)...\tCtrl-L", "Load system (seed and tile set)...")
        self.Bind(wx.EVT_MENU, self.OnLoadSystem, item1)
        itemR = menu.Append(-1, "Reload system (seed and tile set) from files", "Reload system (seed and tile set) from files")
        self.Bind(wx.EVT_MENU, self.OnReloadSystem, itemR)
        item2 = menu.Append(-1, "Save system  (seed and tile set) as...", "Save system (seed and tile set) as...")
        self.Bind(wx.EVT_MENU, self.OnSaveSystemAs, item2)
        item2 = menu.Append(-1, "&Save system (seed and tile set)\tCtrl-S", "Save system (seed and tile set)")
        self.Bind(wx.EVT_MENU, self.OnSaveSystem, item2)
        menu.AppendSeparator()
        
        itemNewAssembly = menu.Append(-1, "New assembly", "New assembly")
        self.Bind(wx.EVT_MENU, self.OnNewAssembly, itemNewAssembly)
        item1 = menu.Append(-1, "Load assembly...", "Load assembly...")
        self.Bind(wx.EVT_MENU, self.OnLoadAssembly, item1)
        itemR = menu.Append(-1, "Reload assembly from file", "Reload assembly from file")
        self.Bind(wx.EVT_MENU, self.OnReloadAssembly, itemR)
        itemR = menu.Append(-1, "Set current assembly as seed", "Set current assembly as seed")
        self.Bind(wx.EVT_MENU, self.OnSetAssemblyAsSeed, itemR)
        itemReset = menu.Append(-1, "&Reset to seed\tCtrl-R", "Reset system to seed")
        self.Bind(wx.EVT_MENU, self.OnResetToSeed, itemReset)
        item2 = menu.Append(-1, "Save assembly as...", "Save assembly as...")
        self.Bind(wx.EVT_MENU, self.OnSaveAssemblyAs, item2)
        item2 = menu.Append(-1, "Save assembly", "Save assembly")
        self.Bind(wx.EVT_MENU, self.OnSaveAssembly, item2)
        menu.AppendSeparator()
        
        itemSVG = menu.Append(-1, "Save assembly as SVG...", "Save assembly as SVG...")
        self.Bind(wx.EVT_MENU, self.OnSaveAssemblyAsSVG, itemSVG)
        itemPDF = menu.Append(-1, "Save assembly as PDF...", "Save assembly as PDF...")
        self.Bind(wx.EVT_MENU, self.OnSaveAssemblyAsPDF, itemPDF)
        menu.AppendSeparator()
        
        item1 = menu.Append(-1, "Load tile set...", "Load tile set...")
        self.Bind(wx.EVT_MENU, self.OnLoadTileSet, item1)
        itemR = menu.Append(-1, "Reload tile set from file", "Reload tile set from file")
        self.Bind(wx.EVT_MENU, self.OnReloadTileSet, itemR)
        itemLoadFromEd = menu.Append(-1, "Load tile set from editor", "Load tile set from editor")
        self.Bind(wx.EVT_MENU, self.OnLoadFromEditor, itemLoadFromEd)
        itemAdd = menu.Append(-1, "Add new tiles to system", "Add new tiles to current system")
        self.Bind(wx.EVT_MENU, self.OnAddToTileSet, itemAdd)
        item2 = menu.Append(-1, "Save tile set as...", "Save tile set as...")
        self.Bind(wx.EVT_MENU, self.OnSaveTileSetAs, item2)
        item2 = menu.Append(-1, "Save tile set", "Save tile set")
        self.Bind(wx.EVT_MENU, self.OnSaveTileSet, item2)

        menu.AppendSeparator()
        item3 = menu.Append(-1, "E&xit\tCtrl-X", "Exit simulator")
        self.Bind(wx.EVT_MENU, self.OnCloseFrame, item3)
        menuBar.Append(menu, "&File")

        #MJP TODO:  Save system as, save system, save assembly as, save assembly, set current assembly as seed, ???

        menuCtrl = wx.Menu()
        
        self.radio2DMode = wx.MenuItem(menuCtrl, -1, text = "2D mode", kind = wx.ITEM_RADIO) 
        self.radio3DMode = wx.MenuItem(menuCtrl, -1, text = "3D mode", kind = wx.ITEM_RADIO)
        menuCtrl.Append(self.radio2DMode)
        menuCtrl.Append(self.radio3DMode)
        self.radio2DMode.Check()
        self.Bind(wx.EVT_MENU, self.OnDimMode, self.radio2DMode)
        self.Bind(wx.EVT_MENU, self.OnDimMode, self.radio3DMode)
        menuCtrl.AppendSeparator()
        temperatureItem = wx.MenuItem(menuCtrl, -1, text = "Set temperature...")
        menuCtrl.Append(temperatureItem)
        self.Bind(wx.EVT_MENU, self.OnSetTemperature, temperatureItem)
        menuCtrl.AppendSeparator()
        self.macroModeMenu = wx.Menu()
        self.checkUseMacroMode = wx.MenuItem(menuCtrl, -1, text = "Use macrotile mode\tCtrl-M", kind = wx.ITEM_CHECK)
        self.macroModeMenu.Append(self.checkUseMacroMode)
        self.Bind(wx.EVT_MENU, self.OnUseMacroMode, self.checkUseMacroMode)
        self.checkUseMacroMode.Check(False)
        
        self.checkOnlyShowMacroMode = wx.MenuItem(menuCtrl, -1, text = "Show only macrotile images (and hide regular tiles)", kind = wx.ITEM_CHECK)
        self.macroModeMenu.Append(self.checkOnlyShowMacroMode)
        self.Bind(wx.EVT_MENU, self.OnOnlyShowMacroMode, self.checkOnlyShowMacroMode)
        self.checkOnlyShowMacroMode.Check(False)

        self.setMacroSize = wx.MenuItem(self.macroModeMenu, -1, "Set macrotile size...")
        self.macroModeMenu.Append(self.setMacroSize)
        self.Bind(wx.EVT_MENU, self.OnSetMacroSize, self.setMacroSize)
        
        self.setMacroCenter = wx.MenuItem(self.macroModeMenu, -1, "Set macrotile center...")
        self.macroModeMenu.Append(self.setMacroCenter)
        self.Bind(wx.EVT_MENU, self.OnSetMacroCenter, self.setMacroCenter)
        
        self.setMacroImages = wx.MenuItem(self.macroModeMenu, -1, "Set macrotile images...")
        self.macroModeMenu.Append(self.setMacroImages)
        self.Bind(wx.EVT_MENU, self.OnSetMacroImages, self.setMacroImages)
        
        menuCtrl.AppendSubMenu(self.macroModeMenu, "Macrotile mode", "Macrotile mode")
        menuCtrl.AppendSeparator()
        breakpointsItem = wx.MenuItem(menuCtrl, -1, text = "Set breakpoints...")
        menuCtrl.Append(breakpointsItem)
        self.Bind(wx.EVT_MENU, self.OnSetBreakpoints, breakpointsItem)
        menuCtrl.AppendSeparator()
        self.radioIgnoreNondet = wx.MenuItem(menuCtrl, -1, text = "Ignore nondeterminism", kind = wx.ITEM_RADIO) 
        self.radioReportNondet = wx.MenuItem(menuCtrl, -1, text = "Report nondeterminism", kind = wx.ITEM_RADIO)
        self.radioStopAtNondet = wx.MenuItem(menuCtrl, -1, text = "Halt on nondeterminism", kind = wx.ITEM_RADIO)
        menuCtrl.Append(self.radioIgnoreNondet)
        menuCtrl.Append(self.radioReportNondet)
        menuCtrl.Append(self.radioStopAtNondet)
        self.radioIgnoreNondet.Check()
        self.Bind(wx.EVT_MENU, self.OnNondetMode, self.radioIgnoreNondet)
        self.Bind(wx.EVT_MENU, self.OnNondetMode, self.radioReportNondet)
        self.Bind(wx.EVT_MENU, self.OnNondetMode, self.radioStopAtNondet)
        menuCtrl.AppendSeparator()
        self.checkKeepCache = wx.MenuItem(menuCtrl, -1, text = "Keep cache for rewinding", kind = wx.ITEM_CHECK)
        menuCtrl.Append(self.checkKeepCache)
        self.checkKeepCache.Check(True)
        self.Bind(wx.EVT_MENU, self.OnKeepCache, self.checkKeepCache)
        menuCtrl.AppendSeparator()
        self.setTilesPerStep = wx.MenuItem(menuCtrl, -1, text = "Set tile additions per step...")
        menuCtrl.Append(self.setTilesPerStep)
        self.Bind(wx.EVT_MENU, self.OnSetTilesPerStep, self.setTilesPerStep)
        menuCtrl.AppendSeparator()
        itemFR = menuCtrl.Append(-1, "Fast rewind", "Fast rewind")
        self.Bind(wx.EVT_MENU, self.OnFastRewind, itemFR)
        itemStepRew = menuCtrl.Append(-1, "Rewind", "Rewind one step")
        self.Bind(wx.EVT_MENU, self.OnRewind, itemStepRew)
        itemStop = menuCtrl.Append(-1, "Pause", "Stop simulation")
        self.Bind(wx.EVT_MENU, self.OnQuit, itemStop)
        itemStep = menuCtrl.Append(-1, "&Forward\tCtrl-F", "Forward one step")
        self.Bind(wx.EVT_MENU, self.OnForward, itemStep)
        itemFF = menuCtrl.Append(-1, "&Go\tCtrl-G", "Fast forward")
        self.Bind(wx.EVT_MENU, self.OnGo, itemFF)
        itemQuit = menuCtrl.Append(-1, "&Quit\tCtrl-Q", "Quit fast forward")
        self.Bind(wx.EVT_MENU, self.OnQuit, itemQuit)
        menuBar.Append(menuCtrl, "&Control")

        self.menuView = wx.Menu()
        
        self.checkViewEditor = wx.MenuItem(self.menuView, -1, text = "View tile type editor", kind = wx.ITEM_CHECK)
        self.menuView.Append(self.checkViewEditor)
        self.Bind(wx.EVT_MENU, self.OnCheckViewEditor, self.checkViewEditor)
        self.checkViewEditor.Check(False)

        self.menuView.AppendSeparator()
        itemCOC = self.menuView.Append(-1, "&Center on coordinates...", "Scroll coordinates to center")
        self.Bind(wx.EVT_MENU, self.OnCenterOnCoords, itemCOC)
        self.menuView.AppendSeparator()
        itemZIn = self.menuView.Append(-1, "Zoom in\tCtrl-+", "Zoom in")
        self.Bind(wx.EVT_MENU, self.OnZoomIn, itemZIn)
        itemZOut = self.menuView.Append(-1, "Zoom out\tCtrl--", "Zoom out")
        self.Bind(wx.EVT_MENU, self.OnZoomOut, itemZOut)
        self.menuView.AppendSeparator()
        self.itemTG = self.menuView.AppendCheckItem(-1, "Display grid lines", "Display grid lines")
        self.Bind(wx.EVT_MENU, self.OnToggleGrid, self.itemTG)
        self.itemOL = self.menuView.Append(-1, "Set outline details", "Specify details relating to outlines")
        self.Bind(wx.EVT_MENU, self.OnSetOutline, self.itemOL)
        self.itemTD = self.menuView.AppendCheckItem(-1, "&Display assembly updates\tCtrl-D", "Display assembly updates")
        self.Bind(wx.EVT_MENU, self.OnDisplayUpdates, self.itemTD)
        self.itemTD.Check(True)
        itemSTT = self.menuView.Append(-1, "Show only matching tile types...", "Set patterns for tile type names to display")
        self.Bind(wx.EVT_MENU, self.OnShowMatching, itemSTT)
        self.menuView.AppendSeparator()
        self.itemShowLabels = self.menuView.AppendCheckItem(-1, "Display tile labels", "Display tile labels")
        self.Bind(wx.EVT_MENU, self.OnDisplayLabels, self.itemShowLabels)
        self.itemShowLabels.Check(True)
        self.itemShowGlues = self.menuView.AppendCheckItem(-1, "Display glue labels", "Display glue labels")
        self.Bind(wx.EVT_MENU, self.OnDisplayGlues, self.itemShowGlues)
        self.itemShowGlues.Check(True)
        self.menuView.AppendSeparator()
        itemBGC = self.menuView.Append(-1, "Set background color", "Set background color")
        self.Bind(wx.EVT_MENU, self.OnSetBackgroundColor, itemBGC)
        self.menuView.AppendSeparator()
        itemAssemblyInfo = self.menuView.Append(-1, "Display assembly info", "Display assembly info")
        self.Bind(wx.EVT_MENU, self.OnDisplayAssemblyInfo, itemAssemblyInfo)
        self.menuView.AppendSeparator()
        itemRefresh = self.menuView.Append(-1, "Refresh screen\tF2", "Refresh screen")
        self.Bind(wx.EVT_MENU, self.OnRefresh, itemRefresh)

        menuBar.Append(self.menuView, "&View")

        menu2 = wx.Menu()
        itemAbout = menu2.Append(-1, "About", "About")
        self.Bind(wx.EVT_MENU, self.OnAbout, itemAbout)
        itemH = menu2.Append(-1, "&Help\tCtrl-H", "Quick help")
        self.Bind(wx.EVT_MENU, self.OnHelp, itemH)
        menuBar.Append(menu2, "&Help")
        itemDoc = menu2.Append(-1, "Documentation\tF1", "Show documentation")
        self.Bind(wx.EVT_MENU, self.OnDoc, itemDoc)
        
        self.frame.SetMenuBar(menuBar)

    def OnChar(self, evt):
        keycode = evt.GetKeyCode()
        if keycode == ord("f") or keycode == ord("F"):
            self.canvas.DoForwardStep()
        elif keycode == ord("b") or keycode == ord("B"):
            self.canvas.DoRewindStep()
        elif keycode == ord("p") or keycode == ord("P"):
            self.canvas.PrintMessage()
        elif keycode == ord("r") or keycode == ord("R"):
            self.canvas.Reset(False)
        elif keycode == ord("s") or keycode == ord("S"):
            self.OnSaveSystem(None)
        elif keycode == ord("h") or keycode == ord("H"):
            self.canvas.ShowHelp()
        elif keycode == ord("l") or keycode == ord("L"):
            self.OnLoadSystem(None)
        elif keycode == ord("=") or keycode == ord("+"):
            self.canvas.ZoomThenRecenter(True)
        elif keycode == ord("-") or keycode == ord("_"):
            self.canvas.ZoomThenRecenter(False)
        elif keycode == ord("g") or keycode == ord("G"):
            self.canvas.DoGo()
        elif keycode == ord("q") or keycode == ord("Q"):
            self.canvas.DoQuit()
        elif keycode == ord("c") or keycode == ord("C"):
            self.OnCenter(None)
        elif keycode == ord("t") or keycode == ord("T"):
            self.canvas.ToggleGrid()
            self.itemTG.Check(self.canvas.drawGrid)
        elif keycode == ord("d") or keycode == ord("D"):
            self.OnDisplayUpdates(None)
        elif keycode == ord("m") or keycode == ord("M"):
            self.canvas.assembly.useMacrotileMode = not self.canvas.assembly.useMacrotileMode
            self.checkUseMacroMode.Check(self.canvas.assembly.useMacrotileMode)
            self.canvas.Refresh()
        elif keycode == wx.WXK_F2:
            self.canvas.Refresh()
        elif keycode == wx.WXK_RETURN:
            self.canvas.DeleteSelection()
        #elif keycode == wx.WXK_DELETE or keycode == wx.WXK_BACK:
            #self.DoDelete()
  
    def OnDoc(self, event):
        if self.docFrame is None:
            self.docFrame = SimUtils.DocumentationFrame(self)
        self.docFrame.SetFocus()

    def OnNewSystem(self, evt):
        bDoIt = True
        if (not self.canvas.assembly is None) and (( self.canvas.assembly.numTiles > 0) or (len(self.canvas.assembly.tileTypes) > 0)):
            bDoIt = False
            dlg = wx.MessageDialog(self.canvas, "Are you sure you want to create a new tile assembly system and lose any unsaved changes?", "Confirm new tile assembly system", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_OK:
                bDoIt = True
            dlg.Destroy()
        if bDoIt == True:
            self.canvas.CreateNewAssembly()
            if not self.editorFrame is None:
                self.editorFrame.canvas.SetTileTypes(self.canvas.assembly.tileTypes)

    def OnReloadSystem(self, evt):
        bDoIt = True
        if (not self.canvas.assembly is None) and ((self.canvas.assembly.numTiles > 0) or (len(self.canvas.assembly.tileTypes) > 0)):
            bDoIt = False
            dlg = wx.MessageDialog(self.canvas, "Are you sure you want to reload the tile assembly system and lose any unsaved changes?", "Confirm reload of tile assembly system", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_OK:
                bDoIt = True
            dlg.Destroy()
        if bDoIt == True:
            if (self.canvas.assembly.systemDefFile is None) or (len(self.canvas.assembly.systemDefFile) == 0):
                wx.MessageDialog(self.frame, "There is no tile assembly system file currently loaded.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
                return False
            else:
                return self.DoLoadSystem(self.canvas.assembly.systemDefFile)
        
    def OnSaveSystem(self, evt):
        if (self.canvas.assembly.systemDefFile is None) or len(self.canvas.assembly.systemDefFile) < 1 or (self.canvas.assembly.tilesetDefFile is None) or len(self.canvas.assembly.tilesetDefFile) < 1:
            self.OnSaveSystemAs(evt)
        bDoIt = True
        if (not self.canvas.assembly is None) and (self.canvas.assembly.numTiles > len(self.canvas.assembly.seedPatch)):
            bDoIt = False
            dlg = wx.MessageDialog(self.canvas, "Please note that this will save the seed assembly (as well as tile set), not necessarily\nthe current assembly. If you'd prefer to save the current assembly, please use the\n'Save assembly' option. Would you like to continue saving the system?", "Confirm saving tile assembly system", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_OK:
                bDoIt = True
            dlg.Destroy()
        if bDoIt == True:
            if (self.canvas.assembly.SaveAssembly(self.canvas.assembly.systemDefFile) == True) and (self.canvas.assembly.SaveTileset(self.canvas.assembly.tilesetDefFile) == True):
                self.OutputMessage("Successfully saved tile assembly system to files " + self.canvas.assembly.systemDefFile + " and " + self.canvas.assembly.tilesetDefFile)
            else:
                self.OutputMessage("Error saving tile assembly system to files " + self.canvas.assembly.systemDefFile + " and " + self.canvas.assembly.tilesetDefFile)

    def OnSaveSystemAs(self, evt):
        bDoIt = True
        if (not self.canvas.assembly is None) and (self.canvas.assembly.numTiles > len(self.canvas.assembly.seedPatch)):
            bDoIt = False
            dlg = wx.MessageDialog(self.canvas, "Please note that this will save the seed assembly (as well as tile set), not necessarily\nthe current assembly. If you'd prefer to save the current assembly, please use the\n'Save assembly' option. Would you like to continue saving the system?", "Confirm saving tile assembly system", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_OK:
                bDoIt = True
            dlg.Destroy()
        if bDoIt == True:
            saveFileDialog = wx.FileDialog(self.frame, "Save system as...", "", "", 
                                           "Assembly system files (*.tdp)|*.tdp",
                                           wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
            saveFileDialog.ShowModal()
            fileName = saveFileDialog.GetPath()
            saveFileDialog.Destroy()
            if len(fileName) > 0:
                tdsFileName = fileName[:-3] + "tds"
                if (self.canvas.assembly.SaveAssembly(fileName) == True) and (self.canvas.assembly.SaveTileset(tdsFileName) == True):
                    self.OutputMessage("Successfully saved tile assembly system to files " + self.canvas.assembly.systemDefFile + " and " + self.canvas.assembly.tilesetDefFile)
            else:
                self.OutputMessage("Error saving tile assembly system to files " + self.canvas.assembly.systemDefFile + " and " + self.canvas.assembly.tilesetDefFile)
        
    def OnNewAssembly(self, evt):
        bDoIt = True
        if (not self.canvas.assembly is None) and (self.canvas.assembly.numTiles > 0):
            bDoIt = False
            dlg = wx.MessageDialog(self.canvas, "Are you sure you want to discard the current assembly?", "Confirm discarding current assembly", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_OK:
                bDoIt = True
            dlg.Destroy()
        if bDoIt == True:
            self.canvas.CreateNewAssembly()
            self.canvas.Refresh()

    def OnSaveAssembly(self, evt):
        if (self.canvas.assembly.systemDefFile is None) or len(self.canvas.assembly.systemDefFile) < 1:
            return self.OnSaveAssemblyAs(evt)
        else:
            if (self.canvas.assembly.tilesetDefFile is None) or len(self.canvas.assembly.tilesetDefFile) < 1:
                wx.MessageDialog(self.frame, "The tile set file must be set before saving the assembly.\nPlease first save the tile set or the entire system.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
                return False
            else:
                return self.canvas.assembly.SaveAssembly(self.canvas.assembly.systemDefFile)

    def OnSaveAssemblyAs(self, evt):
        if (self.canvas.assembly is None) or (self.canvas.assembly.numTiles < 1):
            wx.MessageDialog(self.frame, "There are no tiles in the assembly", "No tiles to save", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return
        elif (self.canvas.assembly.tilesetDefFile is None) or len(self.canvas.assembly.tilesetDefFile) < 1:
            wx.MessageDialog(self.frame, "The tile set file must be set before saving the assembly.\nPlease first save the tile set or the entire system.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return False
        else:
            saveFileDialog = wx.FileDialog(self.frame, "Save assembly as...", "", "", 
                                           "Assembly files (*.tdp)|*.tdp",
                                           wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
            saveFileDialog.ShowModal()
            fileName = saveFileDialog.GetPath()
            saveFileDialog.Destroy()
            if len(fileName) > 0:
                return self.canvas.assembly.SaveAssembly(fileName)
        
    def OnSaveAssemblyAsSVG(self, evt):
        saveFileDialog = wx.FileDialog(self.frame, "Save assembly to SVG as...", "", "", 
                                       "SVG files (*.svg)|*.svg",
                                       wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        saveFileDialog.ShowModal()
        fileName = saveFileDialog.GetPath()
        saveFileDialog.Destroy()
        if len(fileName) > 0:
            # MJP TODO: Ask if frontier locations should be included
            # MJP TODO: Ask if the PDF should show the sequence

            self.canvas.assembly.WriteToPDForSVG(svgFileName=fileName, asSequence=False, displayMask=self.canvas.displayMask)
        
    def OnSaveAssemblyAsPDF(self, evt):
        saveFileDialog = wx.FileDialog(self.frame, "Save assembly to PDF as...", "", "", 
                                       "PDF files (*.pdf)|*.pdf",
                                       wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        saveFileDialog.ShowModal()
        fileName = saveFileDialog.GetPath()
        saveFileDialog.Destroy()
        if len(fileName) > 0:
            # MJP TODO: Ask if frontier locations should be included
            # MJP TODO: Ask if the PDF should show the sequence

            self.canvas.assembly.WriteToPDForSVG(pdfFileName=fileName, asSequence=False, displayMask=self.canvas.displayMask)

    def OnResetToSeed(self, evt):
        self.canvas.Reset(False)
        
    def OnSetAssemblyAsSeed(self, evt):
        bDoIt = True
        if (self.canvas.assembly is None) or (self.canvas.assembly.numTiles < 1):
            bDoIt = False
            dlg = wx.MessageDialog(self.frame, "There are no tiles in the assembly.\nAre you sure you want to set an empty seed?", "No tiles to in assembly", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_OK:
                bDoIt = True
            dlg.Destroy()
        if bDoIt == True:
            self.canvas.assembly.SetCurrentAssemblyAsSeed()

    def OnLoadAssembly(self, evt):
        self.canvas.DoQuit()

        bDoIt = True
        if (not self.canvas.assembly is None) and (self.canvas.assembly.numTiles > 0):
            bDoIt = False
            dlg = wx.MessageDialog(self.canvas, "Are you sure you want to discard the current assembly?", "Confirm discarding current assembly", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_OK:
                bDoIt = True
            dlg.Destroy()
        if bDoIt == True:
            openFileDialog = wx.FileDialog(self.canvas, "Open", "", "", 
                                           "Assembly files (*.tdp)|*.tdp",
                                           wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
            openFileDialog.ShowModal()
            fileName = openFileDialog.GetPath()
            openFileDialog.Destroy()
            if (len(fileName) > 0):
                self.canvas.assembly.ClearSpace()
                return self.DoLoadSystem(fileName, bLoadTileset=False)

    def OnReloadAssembly(self, evt):
        self.canvas.DoQuit()

        bDoIt = True
        if (not self.canvas.assembly is None) and (self.canvas.assembly.numTiles > 0):
            bDoIt = False
            dlg = wx.MessageDialog(self.canvas, "Are you sure you want to discard the current assembly?", "Confirm discarding current assembly", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_OK:
                bDoIt = True
            dlg.Destroy()
        if bDoIt == True:
            if (self.canvas.assembly.systemDefFile is None) or (len(self.canvas.assembly.systemDefFile) == 0):
                wx.MessageDialog(self.frame, "There is no assembly file (.tdp file) currently loaded.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
                return False
            else:
                self.canvas.assembly.ClearSpace()
                return self.DoLoadSystem(self.canvas.assembly.systemDefFile, bLoadTileset=False)
        
    def OnLoadSystem(self, evt):
        self.canvas.DoQuit()

        bDoIt = True
        if (not self.canvas.assembly is None) and (self.canvas.assembly.numTiles > 0):
            bDoIt = False
            dlg = wx.MessageDialog(self.canvas, "Are you sure you want to discard the current system?", "Confirm discarding current system", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_OK:
                bDoIt = True
            dlg.Destroy()
        if bDoIt == True:
            openFileDialog = wx.FileDialog(self.canvas, "Open", "", "", 
                                           "Tile system files (*.tdp)|*.tdp|TAMScript python files (*.py)|*.py",
                                           wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
            openFileDialog.ShowModal()
            fileName = openFileDialog.GetPath()
            openFileDialog.Destroy()
            if (len(fileName) > 0):
                return self.DoLoadSystem(fileName)
            
    def DoLoadSystem(self, fileName, bLoadTileset=True):
        self.canvas.DoQuit()

        dlg = wx.TextEntryDialog(self.canvas, "You can set your random seed below\n(use -1 to choose a new random value):", caption="Set random seed", value=str(self.canvas.randSeed))
        if dlg.ShowModal() == wx.ID_OK:
            self.canvas.randSeed = int(dlg.GetValue())
            dlg.Destroy()
        else:
            dlg.Destroy()
            return False

        self.canvas.SetRandomSeed(self.canvas.randSeed)
        
        if bLoadTileset == True:
            self.OutputMessage("Loading tile assembly system...")
    
            if self.dimMode == SIMU_3D_MODE:
                self.canvas.assembly.seedPatch = []
                self.canvas.Reset(bFromFiles=False, bForce=True)
                self.canvas.assembly = PyTAS3DAssembly()
            else:
                self.canvas.assembly = PyTASAssembly()
            
            ## Determine .tds or .py
            moduleName,fileExtension = os.path.splitext(os.path.split(fileName)[-1])
            self.canvas.assembly.useScript = (fileExtension.lower() == '.py')
            if self.canvas.assembly.useScript: ## script
                print("Using script")
            else:                              ## .tds/.tdp format
                print("Not using script")
                
            midPt = self.canvas.assembly.LoadTileSystem(fileName, backgroundColor=self.canvas.backgroundColor)
            if midPt == SCRIPT_LOAD_FAIL:
                self.OutputMessage("Failed to load script.")
                return False
        
        else:
            self.OutputMessage("Loading assembly...")
            midPt = self.canvas.assembly.LoadTileSystem(fileName, backgroundColor=self.canvas.backgroundColor, bLoadTileset=False)
        
        self.checkUseMacroMode.Check(self.canvas.assembly.useMacrotileMode)         
        
        if midPt == TILESET_LOAD_FAIL_NEEDS_3D:
            dlg = wx.MessageDialog(self.frame, "The selected system requires 3D mode, automatically switching.", caption="Switching to 3D mode")
            if dlg.ShowModal() == wx.ID_OK:
                self.SetDimMode(SIMU_3D_MODE)
                #self.canvas.assembly.temperature = savedAssembly.temperature
                #self.canvas.assembly.tileTypes = savedAssembly.tileTypes
                #self.canvas.assembly.seedPatch = savedAssembly.seedPatch
                self.canvas.Reset(bFromFiles=False, bForce=True)
                midPt = self.canvas.assembly.LoadTileSystem(fileName, backgroundColor=self.canvas.backgroundColor)
                self.canvas.RedrawAssembly()
            else:
                dlg.Destroy()
        elif (self.canvas.assembly.bHas3D == False) and (self.dimMode == SIMU_3D_MODE):
            #dlg = wx.MessageDialog(self.frame, "The selected system does not require 3D mode.\nWould you like to switch to 2D?", caption="Switch to 2D mode?", style=wx.YES_NO)
            #if dlg.ShowModal() == wx.ID_YES:
            print("TODO: Ask about auto switch to 2D when 3D isn't required...")
            #    dlg.Destroy()
        self.canvas.RedrawAssembly(checkAll=True)
        self.canvas.ScrollToCoords(midPt)
        self.canvas.SendStatusBarUpdate(1, "Number of tiles: " + str(self.canvas.assembly.numTiles) + ", frontier size: " + str(len(self.canvas.assembly.frontier)))
        
        if bLoadTileset == True:
            self.OutputMessage("Tile assembly system loaded")
            
            if not self.editorFrame is None:
                self.editorFrame.canvas.SetTileTypes(self.canvas.assembly.tileTypes)
        else:
            self.OutputMessage("Assembly loaded")

        return True
        
    def OnSaveTileSetAs(self, event):
        if (self.canvas.assembly.tileTypes is None) or len(self.canvas.assembly.tileTypes) < 1:
            wx.MessageDialog(self.frame, "There are no tile types to save.", "No tile types to save", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return
        saveFileDialog = wx.FileDialog(self.frame, "Save tile set as...", "", "", 
                                       "Tile set files (*.tds)|*.tds",
                                       wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        saveFileDialog.ShowModal()
        fileName = saveFileDialog.GetPath()
        saveFileDialog.Destroy()
        if len(fileName) > 0:
            if self.canvas.assembly.SaveTileset(fileName) == True:
                self.canvas.assembly.tilesetDefFile = fileName
                self.OutputMessage("Successfully saved tile set to file " + str(self.canvas.assembly.tilesetDefFile))
            
    def OnSaveTileSet(self, event):
        if (self.canvas.assembly.tileTypes is None) or len(self.canvas.assembly.tileTypes) < 1:
            wx.MessageDialog(self.frame, "There are no tile types to save.", "No tile types to save", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return
        if (self.canvas.assembly.tilesetDefFile is None) or len(self.canvas.assembly.tilesetDefFile) < 1:
            self.OnSaveTileSetAs(event)
            return
        if self.canvas.assembly.SaveTileset(self.canvas.assembly.tilesetDefFile) == True:
            self.OutputMessage("Successfully saved tile set to file " + str(self.canvas.assembly.tilesetDefFile))

    def OnLoadTileSet(self, evt):
        bDoIt = True
        if len(self.canvas.assembly.tileTypes) > 0:
            bDoIt = False
            dlg = wx.MessageDialog(self.frame, "Are you sure you want to load a tile set file and lose any unsaved changes?", "Confirm loading tile set", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_OK:
                bDoIt = True
            dlg.Destroy()
        if bDoIt == True:
            openFileDialog = wx.FileDialog(self.frame, "Open", "", "", 
                                           "Tile set files (*.tds)|*.tds",
                                           wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
            openFileDialog.ShowModal()
            fileName = openFileDialog.GetPath()
            openFileDialog.Destroy()
            if (not fileName is None) and len(fileName) > 0:
                tileSet = self.DoLoadTileSet(fileName)
                self.IncorporateNewTileSet(tileSet)
        
    def OnReloadTileSet(self, evt):
        self.canvas.DoQuit()

        if (self.canvas.assembly.tilesetDefFile is None) or (len(self.canvas.assembly.tilesetDefFile) < 1):
            wx.MessageDialog(self.frame, "There is no current tile set file to reload", "No tile set file to reload", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return
        else:
            bDoIt = True
            if len(self.canvas.assembly.tileTypes) > 0:
                bDoIt = False
                dlg = wx.MessageDialog(self.frame, "Are you sure you want to load a tile set file and lose any unsaved changes?", "Confirm loading tile set", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
                if dlg.ShowModal() == wx.ID_OK:
                    bDoIt = True
                dlg.Destroy()
            if bDoIt == True:
                tileSet = self.DoLoadTileSet(self.canvas.assembly.tilesetDefFile)
                self.IncorporateNewTileSet(tileSet)
                
    def DoLoadTileSet(self, fileName):
        self.canvas.DoQuit()
        
        if (len(fileName) > 0):
            assembly = PyTASAssembly()
            loadRet = assembly.LoadTileSet(fileName)
            if loadRet == TILESET_LOAD_FAIL_NEEDS_3D:
                wx.MessageDialog(self.frame, "The new tile set requires 3D mode.\nPlease switch modes before loading.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
                return
            elif loadRet == TILESET_LOAD_SUCCESS:
                self.canvas.assembly.tilesetDefFile = fileName
                bDoIt = True
                if len(assembly.tileTypes) < 1:
                    bDoIt = False
                    dlg = wx.MessageDialog(self.frame, "The loaded tile set is empty.  Do you want to continue?", "Confirm loading empty tile set", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
                    if dlg.ShowModal() == wx.ID_OK:
                        bDoIt = True
                    dlg.Destroy()
            if bDoIt == True:
                return assembly.tileTypes
        
    def OnLoadFromEditor(self, evt):
        if (self.editorFrame is None):
            wx.MessageDialog(self.frame, "There tile type editor is not currently open.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return False
        elif (len(self.editorFrame.canvas.tileTypes) == 0):
            wx.MessageDialog(self.frame, "There are no tile types currently saved in the tile type editor.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return False
        else:
            self.canvas.assembly.tilesetDefFile = None
            self.IncorporateNewTileSet(self.editorFrame.canvas.tileTypes)
            
            
    def IncorporateNewTileSet(self, newTileSet):
            simTTypes = self.canvas.assembly.tileTypes

            # Make a dict out of the new tile set if necessary
            if type(newTileSet) is dict:
                newTTypes = newTileSet
            else:
                newTTypes = dict()
                for ttype in newTileSet:
                    newTTypes[ttype.GetName()] = ttype
            
            # See if there are any current tile types not contained in the new tile set and warn if so
            notFound = []
            for ttypeName in simTTypes:
                if not ttypeName in newTTypes:
                    notFound.append(ttypeName)
            if len(notFound) > 0:
                dlg = wx.MessageDialog(self.canvas, "The current tile set contains tile types not contained in the new tile set.\nContinuing may cause tiles to be deleted from the current assembly.\nDo you want to continue?", "Confirm loading tile set", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
                if not dlg.ShowModal() == wx.ID_OK:
                    return
            else:
                dlg = wx.MessageDialog(self.canvas, "Changing the tile set could make the current assembly unstable.\nDo you want to continue?", "Confirm loading tile set", style=wx.OK|wx.CANCEL|wx.CENTRE|wx.ICON_WARNING)
                if not dlg.ShowModal() == wx.ID_OK:
                    return
            
            # Remove any "not found" tile types from the current assembly
            for i in range(self.canvas.assembly.minX, self.canvas.assembly.maxX+1):
                for j in range(self.canvas.assembly.minY, self.canvas.assembly.maxY+1):
                    loc = self.canvas.assembly.GetLoc((i,j), False, False)
                    if loc is not None:
                        if loc.type.name in notFound:
                            self.canvas.assembly.RemoveTile(loc.coords)
                        else:
                            loc.type = newTTypes[loc.type.name]
                            
            # Swap in the new tile set
            self.canvas.assembly.tileTypes = newTTypes
            self.canvas.assembly.MakeTileSideHashtables()
                            
            # Rebuild the frontier
            self.canvas.assembly.RebuildFrontier()
            
            # MJP: the cache will now be invalid
            
            # And finally, redraw the assembly
            self.canvas.Refresh()

    def OnAddToTileSet(self, event):
        openFileDialog = wx.FileDialog(self.canvas, "Open", "", "", 
                                       "Tile system files (*.tdp)|*.tdp|Tile set files (*.tds)|*.tds",
                                       wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
        openFileDialog.ShowModal()
        fileName = openFileDialog.GetPath()
        openFileDialog.Destroy()
        if (len(fileName) > 0):
            ## Determine .tdp or .tds
            moduleName,fileExtension = os.path.splitext(os.path.split(fileName)[-1])
            if self.dimMode == SIMU_2D_MODE:
                newAssembly = PyTASAssembly()
            else:
                newAssembly = PyTAS3DAssembly()
            if fileExtension.lower() == '.tdp':
                midPt = newAssembly.LoadTileSystem(fileName, backgroundColor=self.canvas.backgroundColor)
                if (newAssembly.bHas3D == True) and (self.dimMode == SIMU_2D_MODE):
                    wx.MessageDialog(self.frame, "The new tiles require 3D mode.  Please switch modes before adding.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
                    return
            else:
                tilesetLoadRet = newAssembly.LoadTileSet(fileName, backgroundColor=self.canvas.backgroundColor)
                if not tilesetLoadRet == TILESET_LOAD_SUCCESS:
                    if (newAssembly.bHas3D == True) and (self.dimMode == SIMU_2D_MODE):
                        wx.MessageDialog(self.frame, "The new tiles require 3D mode.  Please switch modes before adding.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
                        return
                    else:
                        wx.MessageDialog(self.frame, "Error loading tile set file '" + fileName + "'", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
                        return
            self.OutputMessage("Adding tile types...")
            duplicates = []
            dims = 2
            if self.dimMode == SIMU_3D_MODE:
                dims = 3
            for ttName in newAssembly.tileTypes:
                if ttName in self.canvas.assembly.tileTypes:
                    duplicates.append(ttName)
                else:
                    self.canvas.assembly.tileTypes[ttName] = newAssembly.tileTypes[ttName]
                    self.canvas.assembly.tileTypes[ttName].SetDimensions(dims)
            if len(duplicates) > 0:
                self.OutputMessage("Warning:  Duplicate tile type names found and ignored: " + str(duplicates))
            self.OutputMessage("Adding tiles from seed to assembly...")
            self.canvas.assembly.MakeTileSideHashtables()
            for tile in newAssembly.seedPatch:
                loc = self.canvas.assembly.GetLoc(tile[1])
                if loc is not None:
                    self.OutputMessage("Error:  Tile already exists in location " + str(tile[1]) + ". Skipping addition.")
                else:
                    added = self.canvas.assembly.AddTile(self.canvas.assembly.tileTypes[tile[0]], tile[1])
                    if added[0] == False:
                        self.OutputMessage("Error:  Failed to add tile to location " + str(tile[1]))
            self.canvas.SendStatusBarUpdate(1, "Number of tiles: " + str(self.canvas.assembly.numTiles) + ", frontier size: " + str(len(self.canvas.assembly.frontier)))
            self.canvas.RedrawAssembly(checkAll=True)
            self.OutputMessage("Tile addition complete.")

    def OnDisplayUpdates(self, event):
        self.canvas.drawUpdates = (self.canvas.drawUpdates + 1) % 2
        self.itemTD.Check(self.canvas.drawUpdates == SimCanvas.SIMFW_FULLDISPLAY)
        if self.canvas.drawUpdates == SimCanvas.SIMFW_FULLDISPLAY:
            if len(self.canvas.displayPatterns) == 0:
                self.OutputMessage("Switched display mode:  Displaying all tiles in the assembly")
            else:
                self.OutputMessage("Switched display mode:  Displaying all tiles matching the following patterns: " + str(self.canvas.displayPatterns))
            if self.dimMode == SIMU_2D_MODE:
                self.canvas.UpdateSizeVars()
                #self.RedrawAssembly()
                if not self.canvas.savedCenteredCoords is None:
                    self.canvas.CanvasSizeUpdate(self.canvas.savedCenteredCoords)
            elif self.dimMode == SIMU_3D_MODE:
                self.canvas.Refresh()
            self.canvas.SendStatusBarUpdate(1, "Number of tiles: " + str(self.canvas.assembly.numTiles) + ", frontier size: " + str(len(self.canvas.assembly.frontier)))
        #elif self.drawUpdates == SIMFW_GROWTHDISPLAY:
        #    SimUtils.globalApp.OutputMessage("Switched display mode:  Displaying only addition of growth tiles")
        elif self.canvas.drawUpdates == SimCanvas.SIMFW_NODISPLAY:
            if self.dimMode == SIMU_2D_MODE:
                self.canvas.savedCenteredCoords = self.canvas.GetCenterCoords()
            self.OutputMessage("Switched display mode:  No tile additions are being displayed")

    def OnSetTemperature(self, event):
        while True == True:
            dlg = SetTemperatureDialog(self.canvas)
            retVal = dlg.ShowModal()
            if retVal == wx.ID_OK:
                newTemp = dlg.tempCtrl.GetValue()
                if newTemp > 0:
                    self.canvas.assembly.temperature = newTemp
                    self.canvas.assembly.RebuildFrontier()
                    self.canvas.Refresh()
                    self.frame.SetFocus()
                    return
            else:
                self.frame.SetFocus()
                return

    def OnSetOutline(self, event):
        if self.dimMode == SIMU_2D_MODE:
            wx.MessageDialog(self.frame, "This option is for 3D mode only.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
            return
        else:
            dlg = SetOutlineDialog(self)
            dlg.ShowModal()
            return

    def OnDimMode(self, event):
        if self.radio2DMode.IsChecked():
            self.SetDimMode(SIMU_2D_MODE)
        elif self.radio3DMode.IsChecked():
            self.SetDimMode(SIMU_3D_MODE)
        
    def SetDimMode(self, mode):
        if mode == self.dimMode:    # No change
            return
        if (not mode == SIMU_2D_MODE) and (not mode == SIMU_3D_MODE):
            SimUtils.globalApp.OutputMessage("Invalid dimension mode requested: " + str(mode))
            return

        #NOTE: creation of a new canvas re-seeds the random number generator, which may not be desired
        self.dimMode = mode
        
        if self.dimMode == SIMU_2D_MODE:
            self.radio2DMode.Check(True)
            self.SetCaching(True)
            self.checkKeepCache.Enable(True)
        else:
            self.radio3DMode.Check(True)
            self.SetCaching(False)
            self.checkKeepCache.Enable(False)

        pane = self.frame._mgr.GetPane(self.canvas) 
        if pane.IsOk():
            self.canvas.UnsubscribeEventHandlers()
            #self.frame._mgr.ClosePane(pane) 
            self.frame._mgr.DetachPane(pane.window) 
            pane.window.Destroy() 

        self.CreateCanvas()
        self.frame._mgr.AddPane(self.canvas, wx.CENTER, 'Simulation Window')
        self.canvas.SubscribeEventHandlers()
        
        if self.dimMode == SIMU_3D_MODE:
            self.CreateTilesPerStepWin()
        #else:
        #    self.frame._mgr.DetachPane()

        self.frame._mgr.Update()
        
    def CreateStatusBar(self):
        self.frame.CreateStatusBar(3)
        self.frame.SetStatusWidths([140,250,-1])
        
    def CreateTilesPerStepWin(self):
        if self.tilesPerStepPanel is None:
            self.tilesPerStepPanel = wx.Panel(self.frame, -1, wx.DefaultPosition, wx.DefaultSize)
            self.tilesPerStepLabel = wx.StaticText(self.tilesPerStepPanel, -1, "Tiles Per Step", wx.DefaultPosition, wx.Size(50,15))
            self.numTilesPerStepCtrl = wx.lib.intctrl.IntCtrl(self.tilesPerStepPanel, PYTAS_ID_STEPCOUNT, 1, wx.DefaultPosition, wx.Size(50,25))
            self.numTilesPerStepCtrl.SetMin(1)
            self.numTilesPerStepCtrl.SetNoneAllowed(False)
            #self.numTilesPerStepCtrl.SetLimited(True)
            self.numTilesPerStepCtrl.Bind(wx.EVT_ENTER_WINDOW, self.OnMouseEnter)
            self.numTilesPerStepCtrl.Bind(wx.EVT_LEAVE_WINDOW, self.OnMouseLeave)
            self.fullFrontierCheck = wx.CheckBox(self.tilesPerStepPanel, -1, "Full Frontier", wx.DefaultPosition, wx.DefaultSize)
            self.fullFrontierCheck.Bind(wx.EVT_LEAVE_WINDOW, self.OnMouseLeave)
            self.fullFrontierCheck.Bind(wx.EVT_CHECKBOX, self.OnFullFrontierCheck)
            
            #self.numTilesPerStepCtrl.Bind(wx.EVT_CHAR, self.OnTextCtrlChar)
            tsz = wx.BoxSizer(wx.HORIZONTAL)
            tsz.AddSpacer(10)
            tsz.Add(self.tilesPerStepLabel, 1, wx.ALIGN_CENTER_VERTICAL)
            tsz.Add(self.numTilesPerStepCtrl, 1, wx.ALIGN_CENTER_VERTICAL)
            tsz.AddSpacer(10)
            tsz.Add(self.fullFrontierCheck, 1, wx.ALIGN_CENTER_VERTICAL)
            tsz.AddSpacer(10)
            self.tilesPerStepPanel.SetSizerAndFit(tsz)
            self.frame._mgr.AddPane(self.tilesPerStepPanel, wx.aui.AuiPaneInfo().Name("Set Tiles Per Step").ToolbarPane().Top().Row(1).Hide())
            self.frame._mgr.Update()
            
    def OnShowTilesPerStepWin(self, event):
        self.CreateTilesPerStepWin()
        pane = self.frame._mgr.GetPane(self.tilesPerStepPanel)
        if pane.IsShown() == False:
        #if self.itemShowMess.IsChecked():
            pane.Show()
        else:
            pane.Hide()
        self.frame._mgr.Update()

    def OnSetTilesPerStep(self, evt):
        self.OnShowTilesPerStepWin(evt)
        #self.numTilesPerStepCtrl.SetFocus()
        
    def SetNumTilesPerStepEnabled(self, b):
        if self.canvas.stepMode == PYTAS_SM_FULL:
            b = False
        if b:
            self.numTilesPerStepCtrl.SetEditable(True)
            self.numTilesPerStepCtrl.SetBackgroundColour((255, 255, 255))
        else:
            self.numTilesPerStepCtrl.SetEditable(False)
            self.numTilesPerStepCtrl.SetBackgroundColour((200, 200, 200))
            
    def OnFullFrontierCheck(self, evt):
        if self.fullFrontierCheck.IsChecked():
            self.canvas.stepMode = PYTAS_SM_FULL
            self.SetNumTilesPerStepEnabled(False)
            self.OutputMessage("Entering full frontier mode for tile additions per step")
        else:
            self.canvas.stepMode = PYTAS_SM_STEP
            self.SetNumTilesPerStepEnabled(True)
            self.OutputMessage("Leaving full frontier mode for tile additions per step")
        self.frame.SetFocus()

    def OnMouseEnter(self, evt):
        evt.Skip()
        
    def OnMouseLeave(self, evt):
        self.canvas.numTilesPerStep = self.numTilesPerStepCtrl.GetValue()
        if self.canvas.numTilesPerStep < 1:
            self.canvas.numTilesPerStep = 1
        self.frame.SetFocus()
    
    def OnCenter(self, event):
        self.OnCenterOnCoords(event)

    def OnCenterOnCoords(self, event):

        if self.dimMode == SIMU_3D_MODE:
            
            win = CameraPositionWin(self)
            win.ShowModal()
            return

        elif self.dimMode == SIMU_2D_MODE:
            doAgain = True
            coords = (0,0)
            while doAgain == True:
                doAgain = False
                dlg = wx.TextEntryDialog(self.frame, "Enter the coordinates to center on the screen\nin the form (x,y):", caption="Enter coordinates", value=str(coords))
                if not dlg.ShowModal() == wx.ID_OK:
                    return
                result = dlg.GetValue()
                dlg.Destroy()
                try:
                    c = eval(result)
                    coords = (c[0],c[1])
                except:
                    doAgain = True
            self.canvas.ScrollToCoords(coords)
        else:
            return

    def OnDisplayLabels(self, event):
        self.canvas.UpdateTileDisplayMask(bShowLabels=self.itemShowLabels.IsChecked())
        
    def OnDisplayGlues(self, event):
        self.canvas.UpdateTileDisplayMask(bShowGlues=self.itemShowGlues.IsChecked())

    def OnDisplayAssemblyInfo(self, event):
        msg = "The assembly currently has " + str(self.canvas.assembly.numTiles) + " tiles, and tiles at the following minimum and maximum coordinates:"
        self.OutputMessage(msg)
        msg = "minimum X: " + str(self.canvas.assembly.minTiledX) + ", maximum X: " + str(self.canvas.assembly.maxTiledX)
        msg = msg + ", minimum Y: " + str(self.canvas.assembly.minTiledY) + ", maximum Y: " + str(self.canvas.assembly.maxTiledY)
        if self.canvas.assembly.bHas3D == True:
            msg = msg + ", minimum Z: " + str(self.canvas.assembly.minTiledZ) + ", maximum Z: " + str(self.canvas.assembly.maxTiledZ)
        self.OutputMessage(msg)
        
    def OnUseMacroMode(self, event):
        self.canvas.assembly.useMacrotileMode = self.checkUseMacroMode.IsChecked()
        self.canvas.RedrawAssembly(checkAll=True)
    
    def OnOnlyShowMacroMode(self, event):
        self.canvas.assembly.showOnlyMacrotilesMode = self.checkUseMacroMode.IsChecked()
        self.canvas.RedrawAssembly(checkAll=True)

    def OnSetMacroSize(self, event):
        doAgain = True
        macroSize = 2
        if not self.canvas.assembly.macrotileWidth is None:
            macroSize = str(self.canvas.assembly.macrotileWidth) + "," + str(self.canvas.assembly.macrotileHeight)
        while doAgain == True:
            doAgain = False
            dlg = wx.TextEntryDialog(self.frame, "Enter the size of the macrotile rectangles\nas width and height separated by a comma:", caption="Enter macrotile size", value=macroSize)
            if not dlg.ShowModal() == wx.ID_OK:
                return
            result = dlg.GetValue()
            dlg.Destroy()
            try:
                resultList = result.split(',')
                macroWidth = int(resultList[0])
                macroHeight = int(resultList[1])
            except:
                doAgain = True
            if (macroWidth < 1) or (macroHeight < 1):
                doAgain = True
        self.canvas.assembly.macrotileWidth = macroWidth
        self.canvas.assembly.macrotileHeight = macroHeight
        
    def OnSetMacroCenter(self, event):
        doAgain = True
        macroCenter = (0,0)
        if not self.canvas.assembly.macrotileCenter is None:
            macroCenter = self.canvas.assembly.macrotileCenter
        elif not self.canvas.assembly.macrotileWidth is None:
            macroCenter = (int(self.canvas.assembly.macrotileWidth/2), int(self.canvas.assembly.macrotileHeight/2))
        while doAgain == True:
            doAgain = False
            dlg = wx.TextEntryDialog(self.frame, "Enter the center (mapped) location of the macrotile squares\nas a coordinate pair 'x,y':", caption="Enter macrotile center", value=str(macroCenter[0]) + "," + str(macroCenter[1]))
            if not dlg.ShowModal() == wx.ID_OK:
                return
            result = dlg.GetValue()
            dlg.Destroy()
            try:
                coords = result.split(",")
                macroCenter = (int(coords[0]), int(coords[1]))
            except:
                doAgain = True
            if self.canvas.assembly.macrotileWidth is None:
                wx.MessageDialog(self.canvas, "The macrotile size must be set first.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
                doAgain = True
            elif (macroCenter[0] < 0) or (macroCenter[1] < 0) or (macroCenter[0] >= self.canvas.assembly.macrotileWidth) or (macroCenter[1] >= self.canvas.assembly.macrotileHeight):
                wx.MessageDialog(self.canvas, "The coordinates of the macrotile center must be within the bounds of a macrotile.", style=wx.OK|wx.CENTRE|wx.ICON_ERROR).ShowModal()
                doAgain = True
        self.canvas.assembly.macrotileCenter = macroCenter
    
    def OnSetMacroImages(self, event):
        dlg = wx.DirDialog (None, "Please select the directory containing the images for the macrotiles", "Choose image directory", wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST)
        if self.canvas.assembly.macrotileImageDir is not None:
            dlg.SetPath(self.canvas.assembly.macrotileImageDir)
        if not dlg.ShowModal() == wx.ID_OK:
            return
        result = dlg.GetPath()
        dlg.Destroy()
        self.canvas.assembly.macrotileImageDir = result

    def GetVersionString(self):
        return PyTASVersion + " (Simulator Framework version " + SimAppVersion + ")"

    def OnAbout(self, event):
        versionString = self.GetVersionString()
        aboutInfo = wx.adv.AboutDialogInfo()
        aboutInfo.SetName("PyTAS (Python-based Tile Assembly Simulator)")
        aboutInfo.SetVersion(versionString)
        aboutInfo.SetDescription("PyTAS: A Python-based simulator for the abstract Tile Assembly Model")
        #aboutInfo.SetCopyright("(C) 2017-2019")
        aboutInfo.SetWebSite("http://www.self-assembly.net")
        aboutInfo.AddDeveloper("Matthew Patitz (mpatitz@self-assembly.net")
        aboutInfo.AddDeveloper("Daniel Hader")
        aboutInfo.AddDeveloper("Perry Mills")
        wx.adv.AboutBox(aboutInfo)

    #===========================================================================
    # def OnTextCtrlChar(self, evt):
    #     print("getting keycode")
    #     keycode = evt.GetKeyCode()
    #     if keycode == wx.WXK_RETURN:
    #         print("got here")
    #         self.frame.SetFocus()
    #     else:
    #         evt.Skip()
    #         print(str(self.numTilesPerStepCtrl.GetValue()))
    #===========================================================================

#----------------------------------------------------------------------------

app = PyTASApp()
app.MainLoop()

#---------------------------------------------------------------------------


