#!/usr/bin/env python
#----------------------------------------------------------------------------
# Name:         PyTAS3D.py
# Purpose:      A 3D implementation of a Python based version of ISU TAS
#
# Author:       Daniel Hader
#
# Created:      
# RCS-ID:       
# Copyright:    
# Licence:      
#----------------------------------------------------------------------------

"""
This simulator is designed to simulate 3D aTAM systems based on the ISU TAS format

Requirements:
    This application requires Python 2.x due to its dependency on wxPython

Dependencies:
    wxPython Phoenix
        find instructions here:  https://wxpython.org/Phoenix/snapshot-builds/README.txt
        and your download here:  https://wxpython.org/Phoenix/snapshot-builds/
    PyOpenGL
        find instructions here:  http://pyopengl.sourceforge.net/documentation/installation.html
"""

#import wxversion
#wxversion.select('3.0-msw', optionsRequired=True)
#wxversion.select('3.0-Phoenix', optionsRequired=True)

import wx
import wx.xml
import wx.html
import wx.richtext as rt
import ast
import sys
import wx.aui
import wx.lib.intctrl
if sys.version_info[0] < 3:
    from wx.lib.pubsub import pub
else:
    from pubsub import pub
import wx.lib.agw.cubecolourdialog as CCD
import numpy as np

from wx import glcanvas
from PyTAS3DCanvas import *

#---------------------------------------------------------------------------


class RobTileSimApp(wx.App):
    def __init__(self):
        global globalApp
        globalApp = self
        wx.App.__init__(self, redirect=False)

    def OnInit(self):
        self.ctrlPanel = None
        self.messageWin = None
        self.useAdvancedColorPicker = False

        #wx.Log_SetActiveTarget(wx.LogStderr())
        #self.SetAssertMode(assertMode)

        frame = wx.Frame(None, -1, "PyTAS aTAM Simulator", pos=(50,50), size=(400,400),
                         style=wx.DEFAULT_FRAME_STYLE, name="PyTAS aTAM Simulator")
        frame.CreateStatusBar(3)
        frame.SetStatusWidths([-3, -1, -1])
        frame._mgr = wx.aui.AuiManager(frame)

        menuBar = wx.MenuBar()
        menu = wx.Menu()
        item1 = menu.Append(-1, "&Load\tCtrl-L", "Load system")
        self.Bind(wx.EVT_MENU, self.OnLoad, item1)
        itemR = menu.Append(-1, "Reload files", "Reload system from files")
        self.Bind(wx.EVT_MENU, self.OnReload, itemR)
        itemReset = menu.Append(-1, "&Reset to seed\tCtrl-R", "Reset system to seed")
        self.Bind(wx.EVT_MENU, self.OnReset, itemReset)
        item2 = menu.Append(-1, "&Save\tCtrl-S", "Save system")
        self.Bind(wx.EVT_MENU, self.OnSave, item2)
        menu.AppendSeparator()
        item3 = menu.Append(-1, "E&xit\tCtrl-X", "Exit simulator")
        self.Bind(wx.EVT_MENU, self.OnExitApp, item3)
        menuBar.Append(menu, "&File")

        menuCtrl = wx.Menu()
        itemStep = menuCtrl.Append(-1, "&Forward\tCtrl-F", "Forward one step")
        self.Bind(wx.EVT_MENU, self.OnForward, itemStep)
        itemFF = menuCtrl.Append(-1, "&Go\tCtrl-G", "Fast forward")
        self.Bind(wx.EVT_MENU, self.OnGo, itemFF)
        itemQuit = menuCtrl.Append(-1, "&Quit\tCtrl-Q", "Quit fast forward")
        self.Bind(wx.EVT_MENU, self.OnQuit, itemQuit)
        menuBar.Append(menuCtrl, "&Control")

        menuOpt = wx.Menu()
        menuTPS = wx.Menu()
        itemST = menuTPS.AppendRadioItem(-1, "Single Tile", "Add tiles one at a time")
        itemFull = menuTPS.AppendRadioItem(-1, "Full Frontier", "Add the entire frontier")
        self.Bind(wx.EVT_MENU, self.OnSingleTileStep, itemST)
        self.Bind(wx.EVT_MENU, self.OnFullFrontierStep, itemFull)
        menuOpt.AppendSubMenu(menuTPS, "&Tile Additions Per Step", "Number of tile additions Per step")
        menuBar.Append(menuOpt, "&Options")

        menuView = wx.Menu()
        itemSTC = menuView.Append(-1, "&Center\tCtrl+C", "Scroll to center")
        self.Bind(wx.EVT_MENU, self.OnCenter, itemSTC)
        menuView.AppendSeparator()
        itemZIn = menuView.Append(-1, "Zoom in\tCtrl-+", "Zoom in")
        self.Bind(wx.EVT_MENU, self.OnZoomIn, itemZIn)
        itemZOut = menuView.Append(-1, "Zoom out\tCtrl--", "Zoom out")
        self.Bind(wx.EVT_MENU, self.OnZoomOut, itemZOut)
        menuView.AppendSeparator()
        itemTG = menuView.Append(-1, "&Toggle grid lines\tCtrl-T", "Toggle display of grid lines")
        self.Bind(wx.EVT_MENU, self.OnToggleGrid, itemTG)
        itemTD = menuView.Append(-1, "&Toggle display updates\tCtrl-D", "Toggle display of tile additions")
        self.Bind(wx.EVT_MENU, self.OnToggleDisplay, itemTD)
        #itemSTT = menuView.Append(-1, "Show only matching tile types...", "Set patterns for tile type names to display")
        #self.Bind(wx.EVT_MENU, self.OnShowMatching, itemSTT)

        #self.radioShowBitmaps = wx.MenuItem(menuView, -1,text = "Show bitmaps",kind = wx.ITEM_RADIO) 
        #self.radioShowSignals = wx.MenuItem(menuView, -1,text = "Show only signals",kind = wx.ITEM_RADIO) 
        #menuView.Append(self.radioShowBitmaps)
        #menuView.Append(self.radioShowSignals) 
        #self.radioShowBitmaps.Check()
        #self.Bind(wx.EVT_MENU, self.OnShowBitmapsOrLines, self.radioShowBitmaps)
        #self.Bind(wx.EVT_MENU, self.OnShowBitmapsOrLines, self.radioShowSignals)
        menuView.AppendSeparator()
        itemBGC = menuView.Append(-1, "Set background color", "Set background color")
        self.Bind(wx.EVT_MENU, self.OnSetBackgroundColor, itemBGC)
        menuView.AppendSeparator()
        itemRefresh = menuView.Append(-1, "Refresh screen\tF2", "Refresh screen")
        self.Bind(wx.EVT_MENU, self.OnRefresh, itemRefresh)

        menuBar.Append(menuView, "&View")

        menu2 = wx.Menu()
        itemH = menu2.Append(-1, "&Help\tCtrl-H", "Show help")
        self.Bind(wx.EVT_MENU, self.OnHelp, itemH)
        menuBar.Append(menu2, "&Help")
        
        frame.SetMenuBar(menuBar)
        frame.Show(True)

        self.canvas = PyTAS3DCanvas(frame, wx.ID_ANY, frame, wx.DefaultSize, self)
        frame._mgr.AddPane(self.canvas, wx.CENTER, 'Simulation Window')

        self.ctrlPanel = wx.Panel(frame, wx.ID_ANY, pos=wx.DefaultPosition, size=(300,100), style = wx.TAB_TRAVERSAL)
        self.messageWin = rt.RichTextCtrl(self.ctrlPanel, wx.ID_ANY, pos=wx.DefaultPosition, size=(500,70))#, style=wx.NOSCROLL|wx.HSCROLL|wx.NO_BORDER)
        self.messageWin.SetEditable(False)
        sz = wx.BoxSizer(wx.HORIZONTAL)
        sz.Add(self.messageWin, 20, wx.EXPAND, 20)
        self.ctrlPanel.SetSizerAndFit(sz)
        #self.frame._mgr.Bind(wx.aui.EVT_AUI_PANE_CLOSE, self.OnCloseMessagePane)
        frame._mgr.AddPane(self.ctrlPanel, wx.BOTTOM, 'Message Panel')
        frame._mgr.Update()

        frame.Bind(wx.EVT_CLOSE, self.OnCloseFrame)
        frame.Bind(wx.EVT_CHAR, self.canvas.OnChar)
        
        self.toolbar = wx.ToolBar(frame, -1, wx.DefaultPosition, wx.DefaultSize, wx.TB_DOCKABLE | wx.TB_FLAT)
        self.toolbar.SetToolSeparation(20)
        self.toolbar.SetToolBitmapSize(wx.Size(24,24))
        self.toolbar.AddTool(PYTAS_ID_STOP, "Pause", wx.Image("images/pause.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Pause")#, longHelp="Pause simulation")
        self.toolbar.AddTool(PYTAS_ID_FOR, "Forward", wx.Image("images/right.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Forward")#, longHelp="Forward one step")
        self.toolbar.AddTool(PYTAS_ID_FF, "Fast Forward", wx.Image("images/fastright.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Fast Forward")#longHelp="Fast forward")
        self.toolbar.AddSeparator()
        self.toolbar.AddTool(PYTAS_ID_ZOOMIN, "Zoom In", wx.Image("images/zoomin.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Zoom In")#, longHelp="Zoom in")
        self.toolbar.AddTool(PYTAS_ID_ZOOMOUT, "Zoom Out", wx.Image("images/zoomout.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Zoom Out")#, longHelp="Zoom out")
        self.toolbar.AddSeparator()
        self.toolbar.AddTool(PYTAS_ID_ZOOM_MODE, "Zoom Mode", wx.Image("images/zoom.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Zoom Mode", kind=wx.ITEM_RADIO)
        self.toolbar.AddTool(PYTAS_ID_SELECT_MODE, "Select Mode", wx.Image("images/rectangle.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Select Mode", kind=wx.ITEM_RADIO)
        
        btnImg = wx.ArtProvider.GetBitmap(wx.ART_DELETE).ConvertToImage()
        btnImg.Rescale(20, 20)
        self.toolbar.AddTool(PYTAS_ID_DELETE, "Delete Selection", btnImg.ConvertToBitmap(), shortHelp="Delete selected tiles")
        self.toolbar.EnableTool(PYTAS_ID_DELETE, False)
        wx.ArtProvider.GetBitmap(wx.ART_DELETE)
        self.toolbar.AddTool(PYTAS_ID_UNDO, "Undo delete", wx.Image("images/undo.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Undo Delete")
        self.toolbar.EnableTool(PYTAS_ID_UNDO, False)
        self.toolbar.Realize()
        self.toolbar.Bind(wx.EVT_TOOL, self.OnTool)
        frame._mgr.AddPane(self.toolbar, wx.aui.AuiPaneInfo().
                          Name("Simulation toolbar").Caption("Simulation Toolbar").
                          ToolbarPane().Top())
        
        self.tilePanel = wx.Panel(frame, -1, wx.DefaultPosition, wx.DefaultSize)
        self.tileLable = wx.StaticText(self.tilePanel, -1, "Tiles Per Step", wx.DefaultPosition, wx.DefaultSize)
        self.tileTextCtrl = wx.lib.intctrl.IntCtrl(self.tilePanel, PYTAS_ID_STEPCOUNT, 1, wx.DefaultPosition, wx.DefaultSize)
        self.tileTextCtrl.Bind(wx.EVT_TEXT, self.OnKeyTyped)
        tsz = wx.BoxSizer(wx.HORIZONTAL)
        tsz.Add(self.tileLable, 1, wx.FIXED_MINSIZE, 10)
        tsz.Add(self.tileTextCtrl, 1, wx.FIXED_MINSIZE, 10)
        self.tilePanel.SetSizerAndFit(tsz)
        frame._mgr.AddPane(self.tilePanel, wx.aui.AuiPaneInfo().
                          Name("Tile Update").ToolbarPane().Top().Row(1))

        pub.getDefaultPublisher().subscribe(self.DoOutputMessage, "outputMessage")

        # so set the frame to a good size for showing stuff
        frame.SetSize((1200, 800))
        frame.SetFocus()
        self.window = frame

        self.SetTopWindow(frame)
        self.frame = frame
        #wx.Log_SetActiveTarget(wx.LogStderr())
        #wx.Log_SetTraceMask(wx.TraceMessages)
        frame.Maximize()
        
        # tell the manager to 'commit' all the changes just made
        frame._mgr.Update()
        
        self.canvas.SetFocus()

        #self.canvas.InitializeDrawing()
        self.canvas.Refresh()
        #self.canvas.DrawAssemblyUpdate("new")

        return True
    
    def SetTileTextEnabled(self, b):
        if b:
            self.tileTextCtrl.SetEditable(True)
            self.tileTextCtrl.SetBackgroundColour((255, 255, 255))
        else:
            self.tileTextCtrl.SetEditable(False)
            self.tileTextCtrl.SetBackgroundColour((200, 200, 200))

    def OnTool(self, evt):
        if evt.GetId() == PYTAS_ID_STOP:
            
            self.canvas.DoQuit()
        elif evt.GetId() == PYTAS_ID_FOR:
            self.canvas.DoForwardStep()
        elif evt.GetId() == PYTAS_ID_FF:
            self.canvas.DoGo()
        elif evt.GetId() == PYTAS_ID_ZOOMIN:
            self.canvas.ZoomThenRecenter(True)
        elif evt.GetId() == PYTAS_ID_ZOOMOUT:
            self.canvas.ZoomThenRecenter(False)
        elif evt.GetId() == PYTAS_ID_ZOOM_MODE:
            self.SetMouseMode(PYTAS_MODE_ZOOM)
        elif evt.GetId() == PYTAS_ID_SELECT_MODE:
            self.SetMouseMode(PYTAS_MODE_SELECT)
        elif evt.GetId() == PYTAS_ID_DELETE:
            self.canvas.DeleteSelection()
        elif evt.GetId() == PYTAS_ID_UNDO:
            self.UndoDelete()
    
    def OnKeyTyped(self, evt):
        if evt.GetId() == PYTAS_ID_STEPCOUNT:
            text = evt.GetInt()

    def SetMouseMode(self, mode):
        self.canvas.mouseMode = mode
        if mode == PYTAS_MODE_ZOOM:
            self.canvas.rubberBand.reset()
            self.canvas.SetCursor(wx.NullCursor)
            self.toolbar.EnableTool(PYTAS_ID_DELETE, False)
            self.toolbar.EnableTool(PYTAS_ID_UNDO, False)
        else:
            self.toolbar.EnableTool(PYTAS_ID_DELETE, True)
            self.toolbar.EnableTool(PYTAS_ID_UNDO, True)
        
    def UndoDelete(self):
        for loc in self.canvas.lastDeletedList:
            self.canvas.assembly.frontier.discard(loc)
            self.canvas.assembly.AddTile(loc.type, loc.coords)
        self.canvas.lastDeletedList = []
        self.canvas.DoRedrawAssembly()

    def OutputMessage(self, msg):
        wx.CallAfter(pub.getDefaultPublisher().sendMessage, "outputMessage", msg = msg)
        
    def DoOutputMessage(self, msg):
        #if not self.showMessages:
        #    self.messageQueue.append(msg)
        #    return
        if not self.messageWin is None:
            self.messageWin.SetInsertionPointEnd()
            self.messageWin.WriteText("\n" + msg)
            self.messageWin.SetInsertionPointEnd()
            
            if "wxMSW" in wx.PlatformInfo:
                self.messageWin.ScrollLines(self.messageWin.GetNumberOfLines()*5)
                #self.messageWin.ScrollLines(1)
            else:   # Horrible hack to get it to scroll to the bottom on OS X too
                sp = self.messageWin.GetScrollPos(wx.VERTICAL)
                #r = self.messageWin.GetScrollRange(wx.VERTICAL) - self.messageWin.GetScrollThumb(wx.VERTICAL)
                e = wx.ScrollEvent(wx.wxEVT_SCROLLWIN_LINEDOWN,
                                  self.messageWin.GetId(),
                                  10,
                                  wx.VERTICAL)
                for i in range(10):
                    self.messageWin.GetEventHandler().ProcessEvent(e)
            

    def OnExitApp(self, evt):
        dlg = wx.MessageDialog(self.canvas, "Are you sure you want to exit?")
        if dlg.ShowModal() == wx.ID_OK:
            dlg.Destroy()
            self.frame.Close(True)
        dlg.Destroy()

    def OnLoad(self, evt):
        self.canvas.DoLoad()
        
    def OnReload(self, evt):
        self.canvas.Reset(True)
        
    def OnReset(self, evt):
        self.canvas.Reset(False)

    def OnSave(self, event):
        self.canvas.DoSave()
        
    def OnForward(self, event):
        self.canvas.DoForwardStep()
        
    def OnGo(self, event):
        self.canvas.DoGo()
        
    def OnQuit(self, event):
        self.canvas.DoQuit()
        
    def OnToggleGrid(self, event):
        self.canvas.ToggleGrid()
        
    def OnToggleDisplay(self, event):
        self.canvas.ToggleDisplay()
        
    def OnShowMatching(self, event):
        self.canvas.SetShowMatching()
        
    def OnSetBackgroundColor(self, event):
        self.canvas.SetBackgroundColor()

    def OnRefresh(self, event):
        self.canvas.Refresh()
        
    def OnZoomIn(self, event):
        self.canvas.ZoomThenRecenter(True)

    def OnZoomOut(self, event):
        self.canvas.ZoomThenRecenter(False)
        
    def OnCenter(self, event):
        self.canvas.CameraLookAt()

    def OnHelp(self, event):
        self.canvas.ShowHelp()
    
    def OnCloseFrame(self, evt):
        # Make sure the simulation thread is killed
        self.canvas.DoQuit()
        timer = wx.Timer(self)
        timer.Start(1000)
            #self.canvas.goThread.exit()

        # deinitialize the frame manager
        self.frame._mgr.UnInit()
        # delete the frame
        self.frame.Destroy()

    def OnSingleTileStep(self, evt):
        self.canvas.SetStepMode(PYTAS_SM_STEP)

    def OnFullFrontierStep(self, evt):
        self.canvas.SetStepMode(PYTAS_SM_FULL)
#----------------------------------------------------------------------------

app = RobTileSimApp()
app.MainLoop()

#---------------------------------------------------------------------------
